@@ -4,14 +4,13 @@
 #' of the supplied spatial object
 #'
 #' @param x either a Raster*, sf* or Spatial* object
-#' @param data either a Raster*, sf* or Spatial* object
 #' @param map a leaflet or mapview map the extent should be added to. If NULL
 #' standard background layers are created.
-#' @param alpha.regions opacity of the fills or the raster layer(s).
-#' @param layer.name the name of the layer to be shown on the map.
 #' @param popup a \code{list} of HTML strings with the popup contents, usually
 #' created from \code{\link{popupTable}}. See \code{\link{addControl}} for
 #' details.
+#' @param layer.name the name of the layer to be shown on the map.
+#' @param alpha.regions opacity of the fills or the raster layer(s).
 #' @param label a character vector of labels to be shown on mouseover. See
 #' \code{\link{addControl}} for details.
 #' @param ... additional arguments passed on to \code{\link{addRectangles}}
@@ -71,31 +70,18 @@ viewExtent <- function(x,
 
 }
 
-## Add Extent =============================================================
-#' @describeIn viewExtent add extent/bbox of spatial/sf objects to a leaflet map -
-#' This function is deprecated.
-#' Please use leafem::\code{\link[leafem]{addExtent}} instead.
-#'
-#' @export addExtent
-
-addExtent <- function(map, data, ...) {
-
-  .Deprecated(new = "leafem::addExtent", package = "mapview",
-              old = "mapview::addExtent")
-
-  leafem::addExtent(map = map,
-                    data = data,
-                    ...)
-
-  # if (inherits(data, "Spatial")) data = sf::st_as_sfc(data)
-  # x <- sf::st_as_sfc(
-  #   sf::st_bbox(
-  #     checkAdjustProjection(data)
-  #   )
-  # )
-  # m = leafem::addFeatures(x, map = map, ...)
-  # return(m)
-}
+# ## Add Extent =============================================================
+# #' @describeIn viewExtent add extent/bbox of spatial/sf objects to a leaflet map -
+# #' This function is deprecated.
+# #' Please use leafem::\code{\link[leafem]{addExtent}} instead.
+# #'
+# #' @export addExtent
+#
+# addExtent <- function(map, data, ...) {
+#
+#   .Defunct(new = "leafem::addExtent", package = "mapview")
+#
+# }
 
 
 
